#!/bin/bash
###############################################################################
# This library "prepare-system-image-for-read-only-mode" helps to configure 
# an image of Raspberry Pi OS or other compatible Linux distributions 
# to run in read-only mode.
#
# This library is supplementary material to the article 
# "How to configure Raspberry Pi OS to run in read-only mode". 
# The article and this library can be found at https://altomaxtech.com/. 
#
# Copyright (C) 2021, Vladimir Zlatov, Evgueni Tzvetanov, USA 
# Email us at support@altomaxtech.com. 
# 
# This library is free software; you can redistribute it 
# and/or modify it under the terms of the GNU Lesser General Public 
# License as published by the Free Software Foundation; either 
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
# See the GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General 
# Public License along with this library; 
# if not, write to the Free Software Foundation, Inc., 
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
###############################################################################
###############################################################################
# This script was generated automatically by 
# 'prepare-system-image-for-read-only-mode.sh' 
# at Sun 08 Aug 2021 11:05:58 AM UTC.
###############################################################################
###############################################################################
# __RUN_COUNTER__=0
###################
update_run_counter() { 
    # $1 - increment the counter by value

    local key="__RUN_COUNTER__";
    local key_rex="^#\s\+$key=";
    local value_rex="[0-9]\+";
    local eol_rex=".*$";
    local line_rex="$key_rex$value_rex$eol_rex";
    local line=$(grep -e "$line_rex" $0); 
    if [ -n "$line" ] ; then
        local current_count=$(echo "$line" | cut -f2 -d= | sed 's/[^0-9]*//g'); 
        if [ "$1" -gt "0" ] ; then
            local new_count=$((current_count+$1)); 
            sed -i -e "/$line_rex/s/\($key_rex\)$value_rex\($eol_rex\)/\1$new_count\2/" $0; 
        fi
        # return current count value
        echo $current_count;
    else
        # return empty string
        echo "";
    fi
}

check_if_run_allowed__prompt_user_in_doubt__update_counter_on_yes_if_requested() { 
    # $1 - increment the counter by value
    local current_count=$(update_run_counter 0);
    if [ -n "$current_count" ] ; then
        if [ "$current_count" -gt "0" ] ; then
            read -p "This script has already run $current_count time(s). Do you really want to run it one more time? [yes[/]] "  answer
            local yes_pat="^[Yy][Ee][Ss]$";
            if [[ $answer =~ $yes_pat ]] ; then
                if [ "$1" -gt "0" ] ; then
                    update_run_counter "";
                fi
                return 0;
            fi
            
            return 1;
        fi
    fi
    return 0;
}
    
    


is_root() { return $(ls /root/ &>/dev/null); }

if ! is_root; then
    echo "Please run this script as root (use sudo)";
    exit 1;
fi


if ! check_if_run_allowed__prompt_user_in_doubt__update_counter_on_yes_if_requested 0; then
    exit 1;
fi

if apt update --assume-yes ; then
    if apt install --assume-yes ufw; then
        if ufw enable ; then
            if ufw status verbose ; then 
                echo -e "\n\n\n\t\t\t\tUFW installed and enabled SUCCESSFULLY. Reboot in 10 seconds...";
                update_run_counter 1;
                sleep 10;
                reboot;
                exit 0;
            fi
        fi
    fi    
fi

echo "Upgrade FAILED. See the errors above.";
    

###############################################################################
###################  THE END  #################################################
###############################################################################
    

