#!/bin/bash
###############################################################################
# This library "prepare-system-image-for-read-only-mode" helps to configure 
# an image of Raspberry Pi OS or other compatible Linux distributions 
# to run in read-only mode.
#
# This library is supplementary material to the article 
# "How to configure Raspberry Pi OS to run in read-only mode". 
# The article and this library can be found at https://altomaxtech.com/. 
#
# Copyright (C) 2021, Vladimir Zlatov, Evgueni Tzvetanov, USA 
# Email us at support@altomaxtech.com. 
# 
# This library is free software; you can redistribute it 
# and/or modify it under the terms of the GNU Lesser General Public 
# License as published by the Free Software Foundation; either 
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
# See the GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General 
# Public License along with this library; 
# if not, write to the Free Software Foundation, Inc., 
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
###############################################################################
###############################################################################
# This script was generated automatically by 
# 'prepare-system-image-for-read-only-mode.sh' 
# at Sun 08 Aug 2021 11:05:58 AM UTC.
###############################################################################
###############################################################################
# __RUN_COUNTER__=0
###################
update_run_counter() { 
    # $1 - increment the counter by value

    local key="__RUN_COUNTER__";
    local key_rex="^#\s\+$key=";
    local value_rex="[0-9]\+";
    local eol_rex=".*$";
    local line_rex="$key_rex$value_rex$eol_rex";
    local line=$(grep -e "$line_rex" $0); 
    if [ -n "$line" ] ; then
        local current_count=$(echo "$line" | cut -f2 -d= | sed 's/[^0-9]*//g'); 
        if [ "$1" -gt "0" ] ; then
            local new_count=$((current_count+$1)); 
            sed -i -e "/$line_rex/s/\($key_rex\)$value_rex\($eol_rex\)/\1$new_count\2/" $0; 
        fi
        # return current count value
        echo $current_count;
    else
        # return empty string
        echo "";
    fi
}

check_if_run_allowed__prompt_user_in_doubt__update_counter_on_yes_if_requested() { 
    # $1 - increment the counter by value
    local current_count=$(update_run_counter 0);
    if [ -n "$current_count" ] ; then
        if [ "$current_count" -gt "0" ] ; then
            read -p "This script has already run $current_count time(s). Do you really want to run it one more time? [yes[/]] "  answer
            local yes_pat="^[Yy][Ee][Ss]$";
            if [[ $answer =~ $yes_pat ]] ; then
                if [ "$1" -gt "0" ] ; then
                    update_run_counter "";
                fi
                return 0;
            fi
            
            return 1;
        fi
    fi
    return 0;
}
    
    

disable_swap() {
    systemctl stop dphys-swapfile
    systemctl mask dphys-swapfile
    rm /var/swap
}



create_tmpfs_unit_based_on_tmp_mount() {
    # arg 1 - where (mount point dir without closing slash)
    # arg 2 - options (for example, mode=1777,strictatime,nosuid,nodev; or mode=755,relatime,nosuid,nodev)
    #       if empty it is not changed.
    local template_unit_where="/tmp";
    local template_path="/usr/share/systemd/tmp.mount";
    local template_ext="${template_path##*.}"
    local new_unit_dir="/etc/systemd/system";
    local new_unit_where=$1;
    local new_unit_name=$(echo "$new_unit_where" | sed "s/\/*//" | sed "s/\//-/g");
    local new_unit_path="$new_unit_dir/$new_unit_name.$template_ext";
    cp $template_path $new_unit_path

    if [[ ! -z $2 ]]; then 
        sed -i -e "/Description=/s@/tmp@$new_unit_where@" -e "/ConditionPathIsSymbolicLink=/s@/tmp@$new_unit_where@" -e "/Where=/s@/tmp@$new_unit_where@" -e "/Options=/s@Options=.*@Options=$2@"  $new_unit_path; 
    else 
        sed -i -e "/Description=/s@/tmp@$new_unit_where@" -e "/ConditionPathIsSymbolicLink=/s@/tmp@$new_unit_where@" -e "/Where=/s@/tmp@$new_unit_where@" $new_unit_path; 
    fi

    sed -i "1i\
###############################################################################\n\
# This script is a modified copy automatically created\n\
# from the system template '$template_path'\n\
# by '$\(basename "$0"\)'\n\
# at $\(date -u\).\n\
###############################################################################\n" $new_unit_path


}

create_tmpfs_units() {

    create_tmpfs_unit_based_on_tmp_mount "/tmp"      # keep default /tmp mode 'as-is'
    create_tmpfs_unit_based_on_tmp_mount "/var/tmp"  # keep default /tmp mode 'as-is'
    create_tmpfs_unit_based_on_tmp_mount "/var/log" "mode=755,relatime,nosuid,nodev"         # set regular mode
    create_tmpfs_unit_based_on_tmp_mount "/var/lib/dhcp" "mode=755,relatime,nosuid,nodev"    # set regular mode
    create_tmpfs_unit_based_on_tmp_mount "/var/lib/dhcpcd5" "mode=755,relatime,nosuid,nodev" # set regular mode
    create_tmpfs_unit_based_on_tmp_mount "/var/lib/systemd" "mode=755,relatime,nosuid,nodev" # set regular mode
    create_tmpfs_unit_based_on_tmp_mount "/media" "mode=755,relatime,nosuid,nodev"           # set regular mode
    create_tmpfs_unit_based_on_tmp_mount "/mnt" "mode=755,relatime,nosuid,nodev"             # set regular mode
}

enable_tmpfs_units() {

    systemctl enable tmp.mount
    systemctl enable var-tmp.mount
    systemctl enable var-log.mount
    systemctl enable var-lib-dhcp.mount
    systemctl enable var-lib-dhcpcd5.mount
    systemctl enable var-lib-systemd.mount
    systemctl enable media.mount
    systemctl enable mnt.mount
}


is_root() { return $(ls /root/ &>/dev/null); }

if ! is_root; then
    echo "Please run this script as root (use sudo)";
    exit 1;
fi


if ! check_if_run_allowed__prompt_user_in_doubt__update_counter_on_yes_if_requested 0; then
    exit 1;
fi

disable_swap;
create_tmpfs_units;
enable_tmpfs_units;

is_in_graphical_target() { return $(systemctl get-default | grep "graphical.target" &>/dev/null); }

do_switch_to_cli() {

    if is_in_graphical_target; then

        echo "Current read-only mode implementation is not compatible with graphical target.";        
        read -p "Do you want to switch the system to 'Boot to CLI' mode right now? [y/] "  answer
        local yes_pat="^[Yy]$";
        if [[ $answer =~ $yes_pat ]] ; then

            # Switch the system to "Boot to CLI" mode.
            if raspi-config nonint do_boot_behaviour B2 ; then
                echo -e "\n\n\n\t\tThe system has been switched to 'Boot to CLI' mode. Reboot in 10 seconds...";
                update_run_counter 1;
                sleep 10;
                reboot;
            else
                echo "Cannot switch system to 'Boot to CLI' mode. Try to do that manually.";
            fi
        fi
    fi
}

do_switch_to_cli;
    

###############################################################################
###################  THE END  #################################################
###############################################################################
    

