#!/bin/bash
###############################################################################
# This library "prepare-system-image-for-read-only-mode" helps to configure 
# an image of Raspberry Pi OS or other compatible Linux distributions 
# to run in read-only mode.
#
# This library is supplementary material to the article 
# "How to configure Raspberry Pi OS to run in read-only mode". 
# The article and this library can be found at https://altomaxtech.com/. 
#
# Copyright (C) 2021, Vladimir Zlatov, Evgueni Tzvetanov, USA 
# Email us at support@altomaxtech.com. 
# 
# This library is free software; you can redistribute it 
# and/or modify it under the terms of the GNU Lesser General Public 
# License as published by the Free Software Foundation; either 
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
# See the GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General 
# Public License along with this library; 
# if not, write to the Free Software Foundation, Inc., 
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
###############################################################################
###############################################################################
# This script was generated automatically by 
# 'prepare-system-image-for-read-only-mode.sh' 
# at Sun 08 Aug 2021 11:05:58 AM UTC.
###############################################################################
###############################################################################
# __RUN_COUNTER__=0
###################
update_run_counter() { 
    # $1 - increment the counter by value

    local key="__RUN_COUNTER__";
    local key_rex="^#\s\+$key=";
    local value_rex="[0-9]\+";
    local eol_rex=".*$";
    local line_rex="$key_rex$value_rex$eol_rex";
    local line=$(grep -e "$line_rex" $0); 
    if [ -n "$line" ] ; then
        local current_count=$(echo "$line" | cut -f2 -d= | sed 's/[^0-9]*//g'); 
        if [ "$1" -gt "0" ] ; then
            local new_count=$((current_count+$1)); 
            sed -i -e "/$line_rex/s/\($key_rex\)$value_rex\($eol_rex\)/\1$new_count\2/" $0; 
        fi
        # return current count value
        echo $current_count;
    else
        # return empty string
        echo "";
    fi
}

check_if_run_allowed__prompt_user_in_doubt__update_counter_on_yes_if_requested() { 
    # $1 - increment the counter by value
    local current_count=$(update_run_counter 0);
    if [ -n "$current_count" ] ; then
        if [ "$current_count" -gt "0" ] ; then
            read -p "This script has already run $current_count time(s). Do you really want to run it one more time? [yes[/]] "  answer
            local yes_pat="^[Yy][Ee][Ss]$";
            if [[ $answer =~ $yes_pat ]] ; then
                if [ "$1" -gt "0" ] ; then
                    update_run_counter "";
                fi
                return 0;
            fi
            
            return 1;
        fi
    fi
    return 0;
}
    
    


is_root() { return $(ls /root/ &>/dev/null); }

if ! is_root; then
    echo "Please run this script as root (use sudo)";
    exit 1;
fi


if ! check_if_run_allowed__prompt_user_in_doubt__update_counter_on_yes_if_requested 0; then
    exit 1;
fi

modify_fstab_and_reboot() {
    
if [[ -e /etc/fstab.bak0 ]] ; then
    cp /etc/fstab /etc/fstab.bak;
else
    cp /etc/fstab /etc/fstab.bak0;
fi
    if sed -i -e 's/\(^PARTUUID=\S\+\s\+\/boot\s\+vfat\s\+\)\(defaults\)\+\(,\)\?\(r[wo]\|noatime\|noload\)\?\(,\)\?\(r[wo]\|noatime\|noload\)\?\(,\)\?\(r[wo]\|noatime\|noload\)\?\(.*$\)/\1\2,ro,noatime\9/' -e 's/\(^PARTUUID=\S\+\s\+\/\s\+ext4\s\+\)\(defaults\)\+\(,\)\?\(r[wo]\|noatime\|noload\)\?\(,\)\?\(r[wo]\|noatime\|noload\)\?\(,\)\?\(r[wo]\|noatime\|noload\)\?\(.*$\)/\1\2,ro,noatime,noload\9/'  /etc/fstab ; then
        echo -e "\n\n\n\t\tThe system has been switched to 'ro' boot mode. Reboot in 10 seconds...";
        update_run_counter 1;
        sleep 10;
        reboot;
    else
        echo "Cannot modify fstab file to switch to 'ro' boot mode."    
    fi
}

is_in_graphical_target() { return $(systemctl get-default | grep "graphical.target" &>/dev/null); }

report_incompatibility_with_graphical_mode_and_exit() {

    echo "Current read-only mode implementation is not compatible with graphical target. Please switch the system to 'Boot to CLI' mode, then reboot and run this script again.";
        exit 1;
}

if is_in_graphical_target; then

    echo "Current read-only mode implementation is not compatible with graphical target.";        
    read -p "Do you want to switch the system to 'Boot to CLI' mode right now? [y/] "  answer
    local yes_pat="^[Yy]$";
    if [[ $answer =~ $yes_pat ]] ; then

        if raspi-config nonint do_boot_behavior B2 ; then
            modify_fstab_and_reboot;
        else
            echo "Cannot switch system to 'Boot to CLI' mode. Try to do that manually.";
        fi

    else
        report_incompatibility_with_graphical_mode_and_exit;
    fi
else
    modify_fstab_and_reboot;
fi


###############################################################################
###################  THE END  #################################################
###############################################################################
    

